/*****************************************************************************

        CpuLoad.cpp
        Author: Laurent de Soras, 2020

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://www.wtfpl.net/ for more details.

*Tab=3***********************************************************************/



#if defined (_MSC_VER)
	#pragma warning (1 : 4130 4223 4705 4706)
	#pragma warning (4 : 4355 4786 4800)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include "dzy/CpuLoad.h"

#include <algorithm>

#include <cassert>



namespace dzy
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*
==============================================================================
Name: set_sample_freq
Description:
	Sets the sampling rate. Mandatory call before processing anything.
Input parameters:
	- fs: sampling frequency, in Hz. > 0
Throws: Nothing
==============================================================================
*/

void	CpuLoad::set_sample_freq (float fs)
{
	assert (fs > 0);

	_ts = 1.f / fs;

	if (! _init_flag)
	{
		_time_hnd.Init (daisy::TimerHandle::Config {
			daisy::TimerHandle::Config::Peripheral::TIM_2,
			daisy::TimerHandle::Config::CounterDir::UP
		});
		_time_hnd.Start ();

		_init_flag = true;
	}

	const auto     apb1_freq = _time_hnd.GetFreq ();
	_apb1_per = float (1.0 / double (apb1_freq));
}



/*
==============================================================================
Name: blk_beg
Description:
	Call this at the beginning of the profiled callback function.
Throws: Nothing
==============================================================================
*/

void	CpuLoad::blk_beg ()
{
	_t_beg = _time_hnd.GetTick ();
}



/*
==============================================================================
Name: blk_end
Description:
	Call this at the end of the profiled callback function.
Input parameters:
	- nbr_spl: Number of samples processed by the callback function, > 0.
		This is needed to evaluate the time covered by the processing.
Throws: Nothing
==============================================================================
*/

void	CpuLoad::blk_end (int nbr_spl)
{
	assert (_ts > 0);
	assert (nbr_spl > 0);

	const uint32_t t_end      = _time_hnd.GetTick ();
	const uint32_t t_dif      = t_end - _t_beg;
	const float    dur_real_s = nbr_spl * _ts;
	const float    dur_cpu_s  = t_dif * _apb1_per;
	_usage_ratio = dur_cpu_s / dur_real_s;
	_usage_max   = std::max (_usage_ratio, _usage_max);
}



/*
==============================================================================
Name: get_usage
Description:
	Retrieves the CPU usage ratio for the latest processed audio block.
Returns: the usage ratio, in [0 ; 1]
Throws: Nothing
==============================================================================
*/

float	CpuLoad::get_usage () const
{
	return _usage_ratio;
}



/*
==============================================================================
Name: 
Description:
	Retrieves the maximum CPU usage ratio since the last call to this function.
Returns: the usage ratio, in [0 ; 1]
Throws: Nothing
==============================================================================
*/

float	CpuLoad::get_usage_max_and_reset ()
{
	const float    umax = _usage_max;
	_usage_max = _usage_ratio;

	return umax;
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}  // namespace dzy



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
