/*****************************************************************************

        CpuLoad.h
        Author: Laurent de Soras, 2020

Computes the CPU usage in an audio-processing callback function.

General use:

- set_sample_freq() in the init section of your code
- blk_beg() at the beginning of each callback
- blk_end() at the end of each callback

You can then call get_usage() to retrieve the CPU usage of the latest
callback, or get_usage_max_and_reset() to retrieve the maximum CPU usage since
the previous call.

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://www.wtfpl.net/ for more details.

*Tab=3***********************************************************************/



#pragma once
#if ! defined (dzy_CpuLoad_HEADER_INCLUDED)
#define dzy_CpuLoad_HEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <cstdint>

#include "per/tim.h"



namespace dzy
{



class CpuLoad
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	void           set_sample_freq (float fs);
	void           blk_beg ();
	void           blk_end (int nbr_spl);
	float          get_usage () const;
	float          get_usage_max_and_reset ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	// Indicates set_sample_freq() has been called at least once.
	// We use this value to know if we have to initialise the timer.
	bool           _init_flag   = false;

	// Sampling period (1 / Fs), seconds. 0 = not initialised.
	float          _ts          = 0;

	// Timer period, in s. > 0. 0 = not initialised.
	float          _apb1_per    = 0;

	// Handle on the timer
	daisy::TimerHandle
	               _time_hnd;

	// Timer value for the beginning of the block
	uint32_t       _t_beg       = 0;

	// Usage ratio for the latest callback. In [0 ; 1]
	float          _usage_ratio = 0;

	// Maximum usage ratio between two calls to get_usage_max_and_reset().
	float          _usage_max   = 0;



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	bool           operator == (const CpuLoad &other) const = delete;
	bool           operator != (const CpuLoad &other) const = delete;

}; // class CpuLoad



}  // namespace dzy



//#include "dzy/CpuLoad.hpp"



#endif   // dzy_CpuLoad_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

