/*****************************************************************************

        CpuLoadBlink.h
        Author: Laurent de Soras, 2020

Makes a LED blink to show the CPU usage in an audio-processing callback
function.

The LED blinks every second and remains lit for a time calculated as a ratio
of the time spent in the profiled function over the duration of the processed
audio. If a LED period covers more than a single processing call (which is
generally the case), it indicates the maximum CPU usage during this period.

The LED activation function is provided by the caller.

General use:

- set_sample_freq() in the init section of your code
- blk_beg() at the beginning of each callback
- blk_end() at the end of each callback
- If you have to stop displaying the CPU on the LED, call turn_off()

Template parameters:

- LF: LED activation function. Requires:
	ctor and dtor
	void LF::operator () (bool light_flag);
		true to turn the LED on, false to turn it off.

- INV: inversion flag. Indicates that the light shows the remaining CPU ratio
	instead of the CPU usage (which is the default).

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://www.wtfpl.net/ for more details.

*Tab=3***********************************************************************/



#pragma once
#if ! defined (dzy_CpuLoadBlink_HEADER_INCLUDED)
#define dzy_CpuLoadBlink_HEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include "dzy/CpuLoad.h"



namespace dzy
{



template <typename LF>
class CpuLoadBlink
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	LF &           use_led_fnc ();
	void           invert (bool inv_flag);
	void           set_sample_freq (float fs);
	void           blk_beg ();
	void           blk_end (int nbr_spl);
	void           turn_off ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	// Blinking period, in seconds
	static constexpr float  _blink_per_s = 1.f;

	LF             _led_fnc;
	CpuLoad        _load_probe;

	// Flash period, in samples
	int            _period      = 0;

	// Current time position within the period, samples, [0 ; _period[
	int            _pos         = 0;

	// Duration of the light, samples, [0 ; _period]
	int            _duty        = 0;

	// Indicates the current state of the display
	bool           _duty_flag   = false;

	// Indicates if the LED shows the free CPU time
	bool           _invert_flag = false;



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

}; // class CpuLoadBlink



}  // namespace dzy



#include "dzy/CpuLoadBlink.hpp"



#endif   // dzy_CpuLoadBlink_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
