/*****************************************************************************

        CpuLoadBlink.hpp
        Author: Laurent de Soras, 2020

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://www.wtfpl.net/ for more details.

*Tab=3***********************************************************************/



#if ! defined (dzy_CpuLoadBlink_CODEHEADER_INCLUDED)
#define dzy_CpuLoadBlink_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include <cassert>



namespace dzy
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*
==============================================================================
Name: use_led_fnc
Description:
	Gives access to the LED functor
Returns: the LED functor
Throws: Nothing
==============================================================================
*/

template <typename LF>
LF &	CpuLoadBlink <LF>::use_led_fnc ()
{
	return _led_fnc;
}



/*
==============================================================================
Name: invert
Description:
	Sets the display mode: free time or usage.
	Can be called at any moment.
Input parameters:
	- inv_flag: false to indicate usage, true for free time.
Throws: Nothing
==============================================================================
*/

template <typename LF>
void	CpuLoadBlink <LF>::invert (bool inv_flag)
{
	_invert_flag = inv_flag;
}



/*
==============================================================================
Name: set_sample_freq
Description:
	Sets the sampling rate. Mandatory call before processing anything.
Input parameters:
	- fs: sampling frequency, in Hz. > 0
Throws: Nothing
==============================================================================
*/

template <typename LF>
void	CpuLoadBlink <LF>::set_sample_freq (float fs)
{
	_load_probe.set_sample_freq (fs);
	_period = int (fs * _blink_per_s + 0.5f);
	_pos    = 0;
	_duty   = 0;
}



/*
==============================================================================
Name: blk_beg
Description:
	Call this at the beginning of the profiled callback function.
	The function updates the LED status.
Throws: depends on LF
==============================================================================
*/

template <typename LF>
void	CpuLoadBlink <LF>::blk_beg ()
{
	_load_probe.blk_beg ();
	const bool     new_duty_flag = (_pos <= _duty);
	if (new_duty_flag != _duty_flag)
	{
		_led_fnc (new_duty_flag != _invert_flag);
		_duty_flag = new_duty_flag;
	}
}



/*
==============================================================================
Name: blk_end
Description:
	Call this at the end of the profiled callback function.
Input parameters:
	- nbr_spl: Number of samples processed by the callback function, > 0.
		This is needed to evaluate the time covered by the processing.
Throws: Nothing
==============================================================================
*/

template <typename LF>
void	CpuLoadBlink <LF>::blk_end (int nbr_spl)
{
	_pos += nbr_spl;
	if (_pos >= _period)
	{
		_pos -= _period;
		assert (pos < _period);

		const float    cpu_use = _load_probe.get_usage_max_and_reset ();
		_duty = int (cpu_use * float (_period) + 0.5f);
	}

	_load_probe.blk_end (nbr_spl);
}



/*
==============================================================================
Name: turn_off
Description:
	Turns the light off. Call this after having stopped the profiling.
Throws: depends on LF
==============================================================================
*/

template <typename LF>
void	CpuLoadBlink <LF>::turn_off ()
{
	_led_fnc (false);
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}  // namespace dzy



#endif   // dzy_CpuLoadBlink_CODEHEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
